var documenterSearchIndex = {"docs":
[{"location":"overview/#Overview","page":"Overview","title":"Overview","text":"","category":"section"},{"location":"overview/#Application-Structure","page":"Overview","title":"Application Structure","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"A Glimmer application usually contains:","category":"page"},{"location":"overview/","page":"Overview","title":"Overview","text":"Variables - Each variable contains a unique name, a type, and a value. The value of a variable can be changed on the Julia side or on the UI side (using the UI controls). Each variable can be connected to a function that will be called whenever the value is set, either by the UI or Julia.","category":"page"},{"location":"overview/","page":"Overview","title":"Overview","text":"UI Controls - A tree structure that defines the user interface. This structure can contain UI controls such as labels, fields, sliders, buttons etc., and also some components to help layout the controls in different ways, such as containers, expansion panels, tabs etc.","category":"page"},{"location":"overview/","page":"Overview","title":"Overview","text":"Rendering Function - A function that will be called every time any of the application variables value is set. This is a convenient way to take into account all the variables values without connecting each one to a separate event handler.","category":"page"},{"location":"overview/#Example","page":"Overview","title":"Example","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"Let's take a look at the Images example in Glimmer package. This example shows how to display an image created using the Images package, and allow the user to manipulate controls that will update the image. You can run the example using:","category":"page"},{"location":"overview/","page":"Overview","title":"Overview","text":"julia> Glimmer.runExample(\"Images\")","category":"page"},{"location":"overview/","page":"Overview","title":"Overview","text":"(Image: Glimmer Example)","category":"page"},{"location":"overview/#Initilazation","page":"Overview","title":"Initilazation","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"Creating an App object and setting general properties such as the application title and initial window size.","category":"page"},{"location":"overview/","page":"Overview","title":"Overview","text":"using Glimmer, Glimmer.FlexUI\r\nusing Images, ImageSegmentation\r\n\r\napp = App()\r\nprop!(app, :title, \"Glimmer Example - Images\")\r\nprop!(app, :winInitWidth, 1200)\r\nprop!(app, :winInitHeight, 800)","category":"page"},{"location":"overview/#Defining-the-variables","page":"Overview","title":"Defining the variables","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"# image file name\r\nfilename = addVariable!(app, Variable(name=\"filename\", type=\"string\", value=\"C:/t/TestImages/horse.jpg\"))\r\n# size of segments\r\nsize = addVariable!(app, Variable(name=\"size\", type=\"int\", value=10))\r\n# contains the result of the segmentation \r\ndetails = addVariable!(app, Variable(name=\"details\", type=\"string\", value=10))\r\n# source and destination images\r\nsrc_image = addVariable!(app, Variable(name=\"src_image\", type=\"image\", value=\"\"))\r\ndst_image = addVariable!(app, Variable(name=\"dst_image\", type=\"image\", value=\"\"))","category":"page"},{"location":"overview/#Defining-the-UI","page":"Overview","title":"Defining the UI","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"ui = VContainer(\r\n    H1Label(\"Segmantation Example\"),                # Label\r\n    HContainer(\r\n        Button(                                     # Load Button\r\n            text=\"Load a file\",\r\n            fileTypes=\".png,.jpg\",\r\n            variable=\"filename\",\r\n        ),  \r\n        Label(text=\"Current file [\\$(filename)]\"),  # Label showing the current file name\r\n    ),\r\n    Container(\r\n        direction = \"row warp\",\r\n        align = \"space-between center\",\r\n        children = [\r\n            Slider(                                 # Slider to control the segments size\r\n                text=\"Segments Size\",\r\n                trailing_text=\"[\\$()]\",             # slider's label can contain a default \r\n                                                    # variable interpolation for its current value $()\r\n                min=2,\r\n                max=100,\r\n                value=8,\r\n                variable=\"size\"\r\n            ),  \r\n            Label(text=\"\\$(details)\"),              # label showing the segmentation details\r\n                                                    # using the variable interpolation $(details)\r\n        ],\r\n    ),\r\n    H1Label(\"Image Viewer allowing Pan (left-drag) and Zoom (wheel)\"),\r\n    PanZoom(\r\n        style=\"width: 100%; height: 60vh;  border: 1px solid black;\",\r\n        content = Container(\r\n            direction = \"row\",\r\n            children = [\r\n                Card(\r\n                    title=\"Source Image\",\r\n                    # style=card_style, \r\n                    content=VContainer(\r\n                        Image(                      # Source Image       \r\n                            source=\"\\$(src_image)\",\r\n                        ),        \r\n                    ),              \r\n                ),\r\n                Card(\r\n                    title=\"Segmented Image\",\r\n                    # style=card_style, \r\n                    content=VContainer(\r\n                        Image(                      # Segmented Image\r\n                            source=\"\\$(dst_image)\",\r\n                        ),        \r\n                    ),              \r\n                ),\r\n            ]\r\n        ),\r\n    ),        \r\n    \r\n    Glimmer.exampleSourceAsCard(@__FILE__),         # add the source code as the last control\r\n)\r\n# set the controls for the application\r\ncontrols!(app, ui)","category":"page"},{"location":"overview/#The-Rendering-Function-and-others","page":"Overview","title":"The Rendering Function and others","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"# utility function to load the current image from disk\r\nfunction loadImage()\r\n    @info \"Loading Image [$(filename[])]\"\r\n    global src_img = load(filename[])\r\n    src_image[\"png\"] = src_img\r\nend\r\n\r\n# define an event that will trigger when the vriable \"filename\" is changed\r\n# this function is connected to the variable \"filename\" we defined above\r\non(filename) do val\r\n    loadImage()\r\nend\r\n\r\n# the render function will be caulled when any of the UI controls changes value\r\nsrc_img = nothing           \r\nfunction processImages()\r\n    global src_img\r\n\r\n    # performing the segmentation using the variable value size (\"size[]\")\r\n    segments = felzenszwalb(src_img, size[])\r\n    dst_img = map(i->segment_mean(segments,i), labels_map(segments))\r\n    \r\n    # updating the segmented image and the segmentation details variables\r\n    dst_image[\"png\"] = dst_img\r\n    details[] = \"$segments\"\r\nend\r\n\r\n# defining the rendering function that will be called every time a variable's value is updated\r\nfunction render()\r\n    processImages();\r\nend\r\nrenderFunction!(app, render)","category":"page"},{"location":"overview/#Running-the-Application","page":"Overview","title":"Running the Application","text":"","category":"section"},{"location":"overview/","page":"Overview","title":"Overview","text":"# load the initial image and Run the application\r\nloadImage()\r\nrun(app)","category":"page"},{"location":"ref/#Complete-Reference","page":"Reference","title":"Complete Reference","text":"","category":"section"},{"location":"ref/#Glimmer","page":"Reference","title":"Glimmer","text":"","category":"section"},{"location":"ref/","page":"Reference","title":"Reference","text":"Modules = [Glimmer]","category":"page"},{"location":"ref/#Glimmer.examplesFolder-Tuple{}","page":"Reference","title":"Glimmer.examplesFolder","text":"examplesFolder()\n\nreturn the examples folder (~/examples) for this package.\n\n\n\n\n\n","category":"method"},{"location":"ref/#Glimmer.examplesList-Tuple{}","page":"Reference","title":"Glimmer.examplesList","text":"examplesList()\n\nDisplay a list of avalable examples.\n\n\n\n\n\n","category":"method"},{"location":"ref/#Glimmer.rootFolder-Tuple{}","page":"Reference","title":"Glimmer.rootFolder","text":"rootFolder()\n\nreturn the root folder for this package.\n\n\n\n\n\n","category":"method"},{"location":"ref/#Glimmer.runExample-Tuple{String}","page":"Reference","title":"Glimmer.runExample","text":"runExample(example::String)\n\nRun the specific example. [example] is given without extension. For example, if we have an example file named \"Controls.jl\" the command to run it is: Glimmer.runExample(\"Controls\")\n\n\n\n\n\n","category":"method"},{"location":"ref/#UIControls","page":"Reference","title":"UIControls","text":"","category":"section"},{"location":"ref/","page":"Reference","title":"Reference","text":"Modules = [Glimmer.UIControls]","category":"page"},{"location":"ref/#Glimmer.UIControls.Slider","page":"Reference","title":"Glimmer.UIControls.Slider","text":"Slider Control\n\nCreate a slider control.\n\n    text::String = \"\"                   # text to apear to the left of the slider\n    trailing_text::String = \"\"          # text to apear to the right of the slider\n    min::Float64 = 0.0                  # minimum value\n    max::Float64 = 100.0                # maximum value\n    value::Float64 = 0                  # starting value (unused if connected to a variable)\n    step::Float64 = 1.0                 # tick change value\n    variable::Any = nothing             # variable\n\n\n\n\n\n","category":"type"},{"location":"ref/#FlexUI","page":"Reference","title":"FlexUI","text":"","category":"section"},{"location":"ref/","page":"Reference","title":"Reference","text":"Modules = [Glimmer.FlexUI]","category":"page"},{"location":"ref/#Glimmer.FlexUI.forceUpdateControls!-Tuple{App}","page":"Reference","title":"Glimmer.FlexUI.forceUpdateControls!","text":"forceUpdateControls!(app::App)\n\nForce an update of controls that are not updated during the normal execution of an application. Currently, RawHTML is the only     component that behaves like that due to the time it can take to update it.\n\n\n\n\n\n","category":"method"},{"location":"#Glimmer.jl","page":"Home","title":"Glimmer.jl","text":"","category":"section"},{"location":"","page":"Home","title":"Home","text":"Glimmer.jl is a Julia package for creating fast and simple user interface (UI) using the Blink package. The UI is a dynamic Angular application that runs inside a Blink window.","category":"page"},{"location":"","page":"Home","title":"Home","text":"It is designed to allow any Julia user to easily create simple UIs, perform quick code experiments where you can tweak the inputs, and visualize the results in the same window. You can update variable values using the interactive controls or using Julia assinments in the REPL or scripts, and Glimmer will make sure everything is synchronized. ","category":"page"},{"location":"","page":"Home","title":"Home","text":"(Image: Glimmer Example)","category":"page"},{"location":"#Basic-usage:","page":"Home","title":"Basic usage:","text":"","category":"section"},{"location":"","page":"Home","title":"Home","text":"julia> using Glimmer\r\n\r\njulia> Glimmer.runExample(\"Controls\")","category":"page"},{"location":"","page":"Home","title":"Home","text":"(Image: Controls Example)","category":"page"},{"location":"examples/Meshing/#Meshing-Example","page":"Meshing Example","title":"Meshing Example","text":"","category":"section"},{"location":"examples/Meshing/","page":"Meshing Example","title":"Meshing Example","text":"This example demonstrate the use of the 3D viewer and its interaction with other UI controls.","category":"page"},{"location":"examples/Meshing/","page":"Meshing Example","title":"Meshing Example","text":"(Image: Controls Example)","category":"page"}]
}
